@@ -1,4 +1,4 @@
-__version__="4.1.9"
+__version__ = "6.1.2"
 
 # import client
 from aoa.api_client import AoaClient
@@ -9,7 +9,6 @@ from aoa.api.dataset_template_api import DatasetTemplateApi
 from aoa.api.dataset_connection_api import DatasetConnectionApi
 from aoa.api.model_api import ModelApi
 from aoa.api.project_api import ProjectApi
-from aoa.api.trainer_api import TrainerApi
 from aoa.api.trained_model_api import TrainedModelApi
 from aoa.api.trained_model_event_api import TrainedModelEventApi
 from aoa.api.trained_model_artefacts_api import TrainedModelArtefactsApi
@@ -20,8 +19,13 @@ from aoa.api.api_iterator import ApiIterator
 from aoa.api.message_api import MessageApi
 
 # import repo into api package
-from aoa.model_utility.model_utility import ModelUtility
-from aoa.model_utility.evaluate_model import EvaluateModel
-from aoa.model_utility.score_model import ScoreModel
-from aoa.model_utility.train_model import TrainModel
-from aoa.model_utility.base_model import BaseModel
+from aoa.cli.repo_manager import RepoManager
+from aoa.cli.evaluate_model import EvaluateModel
+from aoa.cli.score_model import ScoreModel
+from aoa.cli.train_model import TrainModel
+from aoa.cli.base_model import BaseModel
+
+from aoa.context.model_context import *
+from aoa.util import *
+from aoa.stats.stats import *
+from aoa.stats.store import *
