@@ -23,18 +23,22 @@ class BaseApi(object):
     def _get_header_params(self):
         pass
 
-    def generate_params(self, header_params: List[str], header_vals: List[str]):
+    def generate_params(self, params: List[str], values: List[str]):
         """
-        returns generated parameters
+        returns list of parameters and values as dictionary
 
         Parameters:
-           header_params (List[str]): list of parameter names
-           header_vals (List[str]): list of parameter values
+           params (List[str]): list of parameter names
+           values (List[str]): list of parameter values
 
         Returns:
             (dict): generated parameters
         """
-        return dict(zip(header_params, header_vals))
+
+        # bools in python start with upper case when converted to strs. APIs expect lowercase
+        api_values = [str(v).lower() if type(v) is bool else v for v in values]
+
+        return dict(zip(params, api_values))
 
     def required_params(self, param_names: List[str], dict_obj: Dict[str, str]):
         """
@@ -64,8 +68,8 @@ class BaseApi(object):
         """
         header_params = self._get_header_params()
 
-        query_vars = ['projection', 'sort', 'page', 'size', 'sort']
-        query_vals = [projection, sort, page, size, sort]
+        query_vars = ['projection', 'page', 'size', 'sort']
+        query_vals = [projection, page, size, sort]
         query_params = self.generate_params(query_vars, query_vals)
 
         return self.aoa_client.get_request(
@@ -89,8 +93,8 @@ class BaseApi(object):
         """
         header_params = self._get_header_params()
 
-        query_vars = ['projection', 'sort', 'page', 'size', 'sort', 'archived']
-        query_vals = [projection, sort, page, size, sort, archived]
+        query_vars = ['projection', 'page', 'size', 'sort', 'archived']
+        query_vals = [projection, page, size, sort, archived]
         query_params = self.generate_params(query_vars, query_vals)
 
         return self.aoa_client.get_request(
@@ -125,7 +129,6 @@ class BaseApi(object):
         archives the entity
         Parameters:
            id (str): entity id(uuid) to archive
-           projection (str): projection type
         Returns:
             (dict): entity
         """
@@ -142,7 +145,6 @@ class BaseApi(object):
         unarchives the entity
         Parameters:
            id (str): entity id(uuid) to unarchive
-           projection (str): projection type
         Returns:
             (dict): entity
         """
