@@ -1,20 +1,31 @@
 from __future__ import absolute_import
 
+from requests_oauthlib import OAuth2Session
+from requests.auth import HTTPBasicAuth
+
 import json
 import os
 import yaml
 import logging
 import requests
-from requests.auth import AuthBase
 from typing import List, Dict
 
 
 class AoaClient(object):
     def __init__(self, **kwargs):
         self.logger = logging.getLogger(__name__)
-        self.project_id = None
+        self.project_id = kwargs.get("project_id")
         self.aoa_url = None
-        self.s3_bucket = None
+
+        self.auth_mode = "oauth"
+        self.__auth_user = None
+        self.__auth_pass = None
+        self.__credentials = None
+
+        self.__auth_client_id = None
+        self.__auth_client_secret = None
+        self.__auth_client_token_url = None
+        self.__auth_client_refresh_token = None
 
         self.session = requests.Session()
         self.__parse_aoa_config(**kwargs)
@@ -27,30 +38,20 @@ class AoaClient(object):
     def __parse_kwargs(self, **kwargs):
         self.aoa_url = kwargs.get("aoa_url", self.aoa_url)
 
-        if "auth_mode" in kwargs:
-            auth_mode = kwargs["auth_mode"]
-            if auth_mode == "basic":
-                credentials = kwargs.get("auth_credentials", None)
-                # backward compatibility for credentials key as was called aoa_credentials instead of auth_credentials
-                if "aoa_credentials" in kwargs and credentials is None:
-                    credentials = kwargs.get("aoa_credentials")
+        self.auth_mode = kwargs.get("auth_mode", self.auth_mode)
+        if self.auth_mode == "oauth":
+            self.__auth_client_id = kwargs.get("auth_client_id", self.__auth_client_id)
+            self.__auth_client_secret = kwargs.get("auth_client_secret", self.__auth_client_secret)
+            self.__auth_client_token_url = kwargs.get("auth_client_token_url", self.__auth_client_token_url)
+            self.__auth_client_refresh_token = kwargs.get("auth_client_refresh_token", self.__auth_client_refresh_token)
 
-                self.__configure_basic_auth(credentials,
-                                            kwargs.get("auth_user", None),
-                                            kwargs.get("auth_pass", None))
+        elif self.auth_mode == "oauth-cc":
+            self.__auth_client_id = kwargs.get("auth_client_id", self.__auth_client_id)
+            self.__auth_client_secret = kwargs.get("auth_client_secret", self.__auth_client_secret)
+            self.__auth_client_token_url = kwargs.get("auth_client_token_url", self.__auth_client_token_url)
 
-            elif auth_mode == "kerberos":
-                self.__configure_kerberos()
-
-            elif auth_mode == "oauth":
-                self.__configure_oauth_refresh_token(
-                    kwargs.get("auth_client_id", None),
-                    kwargs.get("auth_client_secret", None),
-                    kwargs.get("auth_client_token_url", None),
-                    kwargs.get("auth_client_refresh_token", None))
-
-            else:
-                raise Exception("Auth mode: {} not supported.".format(auth_mode))
+        else:
+            raise Exception(f"Auth mode: {self.auth_mode} not supported.")
 
         if "verify_connection" in kwargs:
             self.verify_aoa_connection = kwargs["verify_connection"]
@@ -58,26 +59,21 @@ class AoaClient(object):
     def __parse_env_variables(self):
         self.aoa_url = os.environ.get("AOA_URL", self.aoa_url)
 
-        if "AOA_API_AUTH_MODE" in os.environ:
-            auth_mode = os.environ.get("AOA_API_AUTH_MODE")
+        self.auth_mode = os.environ.get("AOA_API_AUTH_MODE", self.auth_mode)
+        if self.auth_mode == "oauth":
+            self.__auth_client_id = os.environ.get("AOA_API_AUTH_CLIENT_ID", self.__auth_client_id)
+            self.__auth_client_secret = os.environ.get("AOA_API_AUTH_CLIENT_SECRET", self.__auth_client_secret)
+            self.__auth_client_token_url = os.environ.get("AOA_API_AUTH_TOKEN_URL", self.__auth_client_token_url)
+            self.__auth_client_refresh_token = os.environ.get("AOA_API_AUTH_REFRESH_TOKEN",
+                                                              self.__auth_client_refresh_token)
 
-            if auth_mode == "basic":
-                self.__configure_basic_auth(os.environ.get("AOA_API_AUTH_CREDENTIALS", None),
-                                            os.environ.get("AOA_API_AUTH_USER", None),
-                                            os.environ.get("AOA_API_AUTH_PASS", None))
+        elif self.auth_mode == "oauth-cc":
+            self.__auth_client_id = os.environ.get("AOA_API_AUTH_CLIENT_ID", self.__auth_client_id)
+            self.__auth_client_secret = os.environ.get("AOA_API_AUTH_CLIENT_SECRET", self.__auth_client_secret)
+            self.__auth_client_token_url = os.environ.get("AOA_API_AUTH_TOKEN_URL", self.__auth_client_token_url)
 
-            elif auth_mode == "kerberos":
-                self.__configure_kerberos()
-
-            elif auth_mode == "oauth":
-                self.__configure_oauth_refresh_token(
-                    os.environ.get("AOA_API_AUTH_CLIENT_ID", None),
-                    os.environ.get("AOA_API_AUTH_CLIENT_SECRET", None),
-                    os.environ.get("AOA_API_AUTH_TOKEN_URL", None),
-                    os.environ.get("AOA_API_AUTH_REFRESH_TOKEN", None))
-
-            else:
-                raise Exception("Auth mode: {} not supported.".format(auth_mode))
+        else:
+            raise Exception(f"Auth mode: {self.auth_mode} not supported.")
 
     def __parse_aoa_config(self, **kwargs):
         if "config_file" in kwargs:
@@ -91,6 +87,23 @@ class AoaClient(object):
         self.__parse_env_variables()
         self.__parse_kwargs(**kwargs)
 
+        if self.auth_mode == "oauth":
+            self.__configure_oauth_refresh_token(
+                self.__auth_client_id,
+                self.__auth_client_secret,
+                self.__auth_client_token_url,
+                self.__auth_client_refresh_token)
+
+        elif self.auth_mode == "oauth-cc":
+            self.__configure_oauth_client_credentials_grant(
+                self.__auth_client_id,
+                self.__auth_client_secret,
+                self.__auth_client_token_url)
+
+    def __validate_url(self):
+        if not self.aoa_url:
+            raise ValueError("Run 'aoa configure' if you have not already (aoa url is not set)")
+
     def set_project_id(self, project_id: str):
         """
         set project id
@@ -118,11 +131,9 @@ class AoaClient(object):
         """
         if not accepts:
             return
+
         accepts = [x.lower() for x in accepts]
-        if 'application/json' in accepts:
-            return 'application/json'
-        else:
-            return ', '.join(accepts)
+        return ', '.join(accepts)
 
     def get_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str]):
         """
@@ -139,6 +150,8 @@ class AoaClient(object):
             raises HTTPError in case of error status code other than 404
         """
 
+        self.__validate_url()
+
         resp = self.session.get(
             url=self.__strip_url(self.aoa_url) + path,
             headers=header_params,
@@ -148,12 +161,7 @@ class AoaClient(object):
         if resp.status_code == 404:
             return None
 
-        resp.raise_for_status()
-
-        try:
-            return resp.json()
-        except ValueError:
-            return resp.text
+        return self.__validate_and_extract_body(resp)
 
     def post_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
         """
@@ -170,6 +178,9 @@ class AoaClient(object):
         Raise:
             raises HTTPError in case of error status code
         """
+
+        self.__validate_url()
+
         resp = self.session.post(
             url=self.__strip_url(self.aoa_url) + path,
             headers=header_params,
@@ -177,12 +188,7 @@ class AoaClient(object):
             data=json.dumps(body)
         )
 
-        resp.raise_for_status()
-
-        try:
-            return resp.json()
-        except ValueError:
-            return resp.text
+        return self.__validate_and_extract_body(resp)
 
     def put_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
         """
@@ -199,6 +205,9 @@ class AoaClient(object):
         Raise:
             raises HTTPError in case of error status code
         """
+
+        self.__validate_url()
+
         resp = self.session.put(
             url=self.__strip_url(self.aoa_url) + path,
             headers=header_params,
@@ -206,12 +215,7 @@ class AoaClient(object):
             data=json.dumps(body)
         )
 
-        resp.raise_for_status()
-
-        try:
-            return resp.json()
-        except ValueError:
-            return resp.text
+        return self.__validate_and_extract_body(resp)
 
     def delete_request(self, path, header_params: Dict[str, str], query_params: Dict[str, str], body: Dict[str, str]):
         """
@@ -226,6 +230,9 @@ class AoaClient(object):
         Raise:
             raises HTTPError in case of error status code
         """
+
+        self.__validate_url()
+
         resp = self.session.delete(
             url=self.__strip_url(self.aoa_url) + path,
             headers=header_params,
@@ -233,7 +240,16 @@ class AoaClient(object):
             data=json.dumps(body)
         )
 
-        resp.raise_for_status()
+        return self.__validate_and_extract_body(resp)
+
+    def __validate_and_extract_body(self, resp):
+        try:
+            resp.raise_for_status()
+        except requests.exceptions.HTTPError as err:
+            if resp.text:
+                raise requests.exceptions.HTTPError(f"Error Message: {resp.text}")
+            else:
+                raise err
 
         try:
             return resp.json()
@@ -243,39 +259,30 @@ class AoaClient(object):
     def __strip_url(self, url):
         return url.rstrip('/')
 
-    def __configure_basic_auth(self, credentials, username, password):
-        if credentials is None and (username is None and password is None):
-            raise Exception("credentials or (username, password) must be defined with auth_mode of 'basic'")
-
-        from base64 import b64encode
-
-        if credentials:
-            self.session.auth = AoaAuth(credentials)
-
-        else:
-            self.session.auth = AoaAuth(b64encode("{}:{}".format(username, password).encode()).decode())
-
     def __configure_oauth_refresh_token(self, client_id, client_secret, token_url, refresh_token):
-        if client_id is None or client_secret is None or token_url is None or refresh_token is None:
-            raise Exception("client_id, client_secret, token_url and refresh_token must be defined "
-                            "with auth_mode of 'oauth'")
+        self.logger.debug("Configuring oauth with refresh token")
 
-        from requests_oauthlib import OAuth2Session
-        from requests.auth import HTTPBasicAuth
+        if client_id is None or client_secret is None or token_url is None or refresh_token is None:
+            raise Exception("Missing CLI configuration.\n" +
+                            "Please (re)copy the CLI configuration from " +
+                            "ModelOps UI -> Session Details -> CLI Config\n")
 
         self.session = OAuth2Session(client_id=client_id)
         self.session.refresh_token(token_url=token_url, refresh_token=refresh_token,
-                                   auth=HTTPBasicAuth(client_id, client_secret))
+                                       auth=HTTPBasicAuth(client_id, client_secret))
 
-    def __configure_kerberos(self):
-        from requests_kerberos import HTTPKerberosAuth, OPTIONAL
-        self.session.auth = HTTPKerberosAuth(mutual_authentication=OPTIONAL)
+    def __configure_oauth_client_credentials_grant(self, client_id, client_secret, token_url):
+        self.logger.debug("Configuring oauth with client credentials grant")
 
+        if client_id is None or client_secret is None or token_url is None:
+            raise Exception("AOA_API_AUTH_CLIENT_ID, AOA_API_AUTH_CLIENT_SECRET, "
+                            "AOA_API_AUTH_TOKEN_URL must be defined "
+                            "with AOA_API_AUTH_MODE of 'oauth-cc (client-credentials)'")
 
-class AoaAuth(AuthBase):
-    def __init__(self, aoa_credentials):
-        self.aoa_credentials = aoa_credentials
+        from requests_oauthlib import OAuth2Session
+        from requests.auth import HTTPBasicAuth
+        from oauthlib.oauth2 import BackendApplicationClient
 
-    def __call__(self, r):
-        r.headers['Authorization'] = "Basic {}".format(self.aoa_credentials)
-        return r
+        self.session = OAuth2Session(client=BackendApplicationClient(client_id=client_id))
+        self.session.fetch_token(token_url=token_url,
+                                 auth=HTTPBasicAuth(client_id, client_secret))
