@@ -3,15 +3,16 @@ import numpy as np
 import pandas as pd
 import os
 import math
+import logging
 
 from typing import List, Dict
 from teradataml.analytics.valib import *
-from teradataml import configure
-from teradataml.dataframe.dataframe import DataFrame
+from teradataml import DataFrame
 from decimal import Decimal
 from aoa.stats.metrics import publish_data_stats
+from aoa.context.model_context import ModelContext
 
-configure.val_install_location = os.environ.get("AOA_VAL_DB", "VAL")
+logger = logging.getLogger(__name__)
 
 
 class _NpEncoder(json.JSONEncoder):
@@ -26,53 +27,80 @@ class _NpEncoder(json.JSONEncoder):
             return super(_NpEncoder, self).default(obj)
 
 
-def _get_reference_edges(variables, statistics, dtypes, bins=10, **kwargs):
+def _get_legacy_reference_edges(variables: List[str], statistics, dtypes):
     edges = []
+
     if os.path.isfile("artifacts/input/data_stats.json"):
         with open("artifacts/input/data_stats.json") as f:
             data_stats = json.load(f)
-
             training_stats = dict(data_stats["features"])
             training_stats.update(data_stats["predictors"])
-
             for v in variables:
                 edges.append(training_stats[v]["statistics"]["histogram"]["edges"])
-
     else:
-        # should return what is in the feature catalog.. for now calculate linspace boundaries based on min/max
-        ranges = statistics.drop(statistics.columns.difference(["xcol", "xmin", "xmax"]), axis=1)
-        ranges = ranges.set_index("xcol")
-        ranges = ranges.to_dict(orient='index')
-
-        for var in variables:
-            x_min, x_max = ranges[var]["xmin"], ranges[var]["xmax"]
-
-            # if integer type and range is less than the number of bins, only use 'range' bins
-            if x_max - x_min < bins and dtypes[var].startswith("int"):
-                edges.append(np.linspace(x_min, x_max, int(x_max) - int(x_min) + 1).tolist())
-            # if decimal fix to two decimal places for now.. we really need to know the decimal precision to do this
-            # correctly.
-            elif dtypes[var].startswith("decimal") or dtypes[var].startswith("float"):
-                # for bins other than 1st and last, round to two decimal places
-                # (min / max must be rounded up / down accordingly so easier to just use the vals from stats)
-                vals = np.linspace(x_min, x_max, bins + 1).tolist()
-
-                for i in range(1, bins):
-                    vals[i] = Decimal("{:.2f}".format(vals[i]))
-
-                # VAL Histogram function fails for some reason when FLOAT min boundary is set to min value, so rounding min down
-                # max is rounded up just for symmetry (max boundary set to max value does work with VAL)
-                p = 2
-                vals[0] = np.true_divide(np.floor(vals[0] * 10 ** p), 10 ** p)
-                vals[bins] = np.true_divide(np.ceil(vals[bins] * 10 ** p), 10 ** p)
-
-                edges.append(vals)
-            else:
-                edges.append(np.linspace(x_min, x_max, bins + 1).tolist())
+        edges = _compute_continuous_edges(variables, statistics, dtypes)
 
     return edges
 
 
+def _get_reference_edges(variables: List[str], feature_metadata_fqtn: str):
+    from aoa.stats import store
+
+    stats = store.get_feature_stats(feature_metadata_fqtn)
+
+    return _decode_continuous_edges_from_stats(variables, stats)
+
+
+def _compute_continuous_edges(variables, statistics, dtypes, bins=10, **kwargs):
+    edges = []
+
+    # should return what is in the feature catalog.. for now calculate linspace boundaries based on min/max
+    ranges = statistics.drop(statistics.columns.difference(["xcol", "xmin", "xmax"]), axis=1)
+    ranges = ranges.set_index("xcol")
+    ranges.index = ranges.index.map(str.lower)
+    ranges = ranges.to_dict(orient='index')
+
+    for var in variables:
+        x_min, x_max = ranges[var]["xmin"], ranges[var]["xmax"]
+
+        # if integer type and range is less than the number of bins, only use 'range' bins
+        if x_max - x_min < bins and dtypes[var].startswith("int"):
+            edges.append(np.linspace(x_min, x_max, int(x_max) - int(x_min) + 1).tolist())
+        # if decimal fix to two decimal places for now.. we really need to know the decimal precision to do this
+        # correctly.
+        elif dtypes[var].startswith("decimal") or dtypes[var].startswith("float"):
+            # for bins other than 1st and last, round to two decimal places
+            # (min / max must be rounded up / down accordingly so easier to just use the vals from stats)
+            vals = np.linspace(x_min, x_max, bins + 1).tolist()
+
+            for i in range(1, bins):
+                vals[i] = float(Decimal("{:.2f}".format(vals[i])))
+
+            # VAL Histogram function fails for some reason when FLOAT min boundary is set to min value, so rounding min down
+            # max is rounded up just for symmetry (max boundary set to max value does work with VAL)
+            p = 2
+            vals[0] = np.true_divide(np.floor(vals[0] * 10 ** p), 10 ** p)
+            vals[bins] = np.true_divide(np.ceil(vals[bins] * 10 ** p), 10 ** p)
+
+            edges.append(vals)
+        else:
+            edges.append(np.linspace(x_min, x_max, bins + 1).tolist())
+
+    return edges
+
+
+def _decode_continuous_edges_from_stats(variables, statistics):
+    new_edges = []
+    for v in variables:
+        if statistics[v.lower()]['edges']:
+            new_edges.append(statistics[v.lower()]['edges'])
+        else:
+            raise Exception(f"Failed to find variable {v} edges in Dataset Stats Metadata. " +
+                            f"Have you run `aoa feature compute-stats` for this columns?")
+
+    return new_edges
+
+
 def _convert_all_edges_to_val_str(all_edges):
     # boundaries for multiple columns follows the following format..
     # ["{10, 0, 200000}", "{5, 0, 100}"]
@@ -104,12 +132,9 @@ def _dict_unmapper(obj, mapper):
     return {_get_key_of(mapper, k) if _get_key_of(mapper, k) is not None else k: v for (k, v) in obj.items()}
 
 
-def _list_mapper(obj, mapper):
-    return [mapper[item] if item in mapper else item for item in obj] if mapper else obj
-
-
 def _list_unmapper(obj, mapper):
-    return [_get_key_of(mapper, item) if _get_key_of(mapper, item) is not None else item for item in obj] if mapper else obj
+    return [_get_key_of(mapper, item) if _get_key_of(mapper, item) is not None else item for item in
+            obj] if mapper else obj
 
 
 def _fill_missing_bins(bin_edges, bin_values, var_ref_edges):
@@ -129,7 +154,8 @@ def _strip_key_x(d: Dict):
 
 
 def _process_category_labels(labels, overrides):
-    return {k: _stringify_keys(overrides)[v] if v in _stringify_keys(overrides).keys() else v for (k, v) in _stringify_keys(labels).items()}
+    return {k: _stringify_keys(overrides)[v] if v in _stringify_keys(overrides).keys() else v for (k, v) in
+            _stringify_keys(labels).items()}
 
 
 def _process_category_dictionary(labels, current=[]):
@@ -157,7 +183,8 @@ def _process_frequencies_dictionary(frequencies_dict):
     return result
 
 
-def _process_categorical_var(frequencies, group_label, var, importance, category_labels_override, is_ordinal, category_dictionary):
+def _process_categorical_var(frequencies, group_label, var, importance, category_labels_override, is_ordinal,
+                             category_dictionary):
     data_struct = {
         "type": "categorical",
         "group": group_label,
@@ -248,34 +275,35 @@ def _process_continuous_var(hist, stats, var_ref_edges, group_label, var, import
     return data_struct
 
 
-def _parse_scoring_stats(features_df: DataFrame, predicted_df: DataFrame, importance_dict=None, **kwargs):
+def _parse_scoring_stats(features_df: DataFrame,
+                         predicted_df: DataFrame,
+                         data_stats: Dict,
+                         importance_dict: Dict = None,
+                         feature_metadata_fqtn: str = None,
+                         feature_metadata_group: str = None):
     if not isinstance(features_df, DataFrame):
         raise TypeError("We only support teradataml DataFrame for features")
 
     if not isinstance(predicted_df, DataFrame):
         raise TypeError("We only support teradataml DataFrame for predictions")
 
-    with open("artifacts/input/data_stats.json", 'r') as f:
-        data_struct = json.load(f)
-
     features = []
-    predictors = []
+    targets = []
     categorical = []
     category_labels = {}
     category_ordinals = {}
     category_dictionary = {}
     importance = {}
-    edges = {}
-    feature_group = "default"
-    predictor_group = "default"
-    mapper = kwargs.get("mapper", None)
 
     for var_type in ["features", "predictors"]:
-        for name, value in data_struct[var_type].items():
+        for name, value in data_stats[var_type].items():
+            # for backward compatibility with data stats created before we lower-cased
+            name = name.lower()
+
             if var_type == "features":
                 features.append(name)
             elif var_type == "predictors":
-                predictors.append(name)
+                targets.append(name)
             if "type" in value and value["type"] == "categorical":
                 categorical.append(name)
                 if "category_dictionary" in value:
@@ -284,15 +312,8 @@ def _parse_scoring_stats(features_df: DataFrame, predicted_df: DataFrame, import
                     category_labels[name] = value["category_labels"]
                 if "ordinal" in value and value["ordinal"]:
                     category_ordinals[name] = value["ordinal"]
-            if "statistics" in value and "histogram" in value["statistics"] and "edges" in value["statistics"]["histogram"]:
-                edges[name] = value["statistics"]["histogram"]["edges"]
             if "importance" in value:
                 importance[name] = value["importance"]
-            if "group" in value:
-                if var_type == "features":
-                    feature_group = value["group"]
-                elif var_type == "predictors":
-                    predictor_group = value["group"]
 
     total_rows_features = features_df.shape[0]
     total_rows_predictors = predicted_df.shape[0]
@@ -300,70 +321,94 @@ def _parse_scoring_stats(features_df: DataFrame, predicted_df: DataFrame, import
     if total_rows_predictors != total_rows_features:
         raise ValueError("The number of predictions do not match the number of features!")
 
-    data_struct = _capture_stats(features_df,
-                                 features,
-                                 [],
-                                 categorical,
-                                 category_labels,
-                                 category_ordinals,
-                                 category_dictionary,
-                                 importance_dict if importance_dict is not None else importance,
-                                 feature_group,
-                                 predictor_group,
-                                 mapper=mapper)
+    data_stats = _capture_stats(df=features_df,
+                                features=features,
+                                targets=[],
+                                categorical=categorical,
+                                category_labels=category_labels,
+                                category_ordinals=category_ordinals,
+                                category_dictionary=category_dictionary,
+                                importance=importance_dict if importance_dict is not None else importance,
+                                feature_metadata_fqtn=feature_metadata_fqtn,
+                                feature_metadata_group=feature_metadata_group)
 
-    predictors_struct = _capture_stats(predicted_df,
-                                       [],
-                                       predictors,
-                                       categorical,
-                                       category_labels,
-                                       category_ordinals,
-                                       category_dictionary,
-                                       importance_dict if importance_dict is not None else importance,
-                                       feature_group,
-                                       predictor_group,
-                                       mapper=mapper)
+    predictors_struct = _capture_stats(df=predicted_df,
+                                       features=[],
+                                       targets=targets,
+                                       categorical=categorical,
+                                       category_labels=category_labels,
+                                       category_ordinals=category_ordinals,
+                                       category_dictionary=category_dictionary,
+                                       importance=importance_dict if importance_dict is not None else importance,
+                                       feature_metadata_fqtn=feature_metadata_fqtn,
+                                       feature_metadata_group=feature_metadata_group)
 
-    data_struct["predictors"] = predictors_struct["predictors"]
+    data_stats["predictors"] = predictors_struct["predictors"]
 
-    return data_struct
+    return data_stats
 
 
 def _capture_stats(df: DataFrame,
                    features: List,
-                   predictors: List,
+                   targets: List,
                    categorical: List,
                    category_labels: Dict = {},
                    category_ordinals: Dict = {},
                    category_dictionary: Dict = {},
                    importance: Dict = {},
-                   feature_group="default",
-                   predictor_group="default",
-                   **kwargs):
-
+                   feature_metadata_fqtn: str = None,
+                   feature_metadata_group: str = "default"):
     if not isinstance(df, DataFrame):
-        raise TypeError("We only support teradataml DataFrame currently")
+        raise TypeError("We only support teradataml DataFrame")
+
+    if not feature_metadata_group:
+        raise ValueError("feature_metadata_group parameter must be defined")
+
+    # lowercase all keys/names to avoid mismatches between sql/dataframes case sensitivity
+    features = [f.lower() for f in features]
+    targets = [t.lower() for t in targets]
+    categorical = [c.lower() for c in categorical]
+    category_labels = {k.lower(): v for k, v in category_labels.items()}
+    category_ordinals = {k.lower(): v for k, v in category_ordinals.items()}
+    category_dictionary = {k.lower(): v for k, v in category_dictionary.items()}
+    importance = {k.lower(): v for k, v in importance.items()}
+    df_columns = [c.lower() for c in df.columns]
+
+    # validate that the dataframe contains the features/targets provided
+    if features and not set(features).issubset(df_columns):
+        raise ValueError(f"Dataframe with columns ({df.columns}) does not contain features: {features}")
+
+    if targets and not set(targets).issubset(df_columns):
+        raise ValueError(f"Dataframe with columns ({df.columns}) does not contain targets: {targets}")
 
     total_rows = df.shape[0]
-    dtypes = {r[0]: r[1] for r in df.dtypes._column_names_and_types}
-    continuous_vars = list((set(features) | set(predictors)) - set(categorical))
-    categorical_vars = list((set(features) | set(predictors)) - set(continuous_vars))
+    continuous_vars = list((set(features) | set(targets)) - set(categorical))
+    categorical_vars = list((set(features) | set(targets)) - set(continuous_vars))
     reference_edges = []
-    mapper = kwargs.get('mapper', {})
 
     if len(continuous_vars) > 0:
-        continuous_vars_mapped = _list_mapper(continuous_vars, mapper)
-        stats = valib.Statistics(data=df, columns=','.join(continuous_vars_mapped), stats_options="all")
+        stats = valib.Statistics(data=df, columns=continuous_vars, stats_options="all")
         stats = stats.result.to_pandas().reset_index()
-        reference_edges = _get_reference_edges(continuous_vars, stats, dtypes)
-        hist = valib.Histogram(data=df, columns=','.join(continuous_vars_mapped),
+        stats["xcol"] = stats["xcol"].str.lower()
+
+        if feature_metadata_fqtn:
+            reference_edges = _get_reference_edges(continuous_vars, feature_metadata_fqtn)
+        else:
+            logging.warning(
+                "This dataset doesn't have feature metadata information, this is deprecated behaviour and will not be "
+                "supported in next release; at the moment histogram edges are computed dynamically during training")
+            dtypes = {r[0].lower(): r[1] for r in df.dtypes._column_names_and_types}
+            reference_edges = _get_legacy_reference_edges(continuous_vars, stats, dtypes)
+
+        hist = valib.Histogram(data=df, columns=continuous_vars,
                                boundaries=_convert_all_edges_to_val_str(reference_edges))
         hist = hist.result.to_pandas().reset_index()
+        hist["xcol"] = hist["xcol"].str.lower()
 
     if len(categorical_vars) > 0:
-        categorical_vars_mapped = _list_mapper(categorical_vars, mapper)
-        frequencies = valib.Frequency(data=df, columns=','.join(categorical_vars_mapped))
+        frequencies = valib.Frequency(data=df, columns=categorical_vars)
         frequencies = frequencies.result.to_pandas().reset_index()
+        frequencies["xcol"] = frequencies["xcol"].str.lower()
 
     data_struct = {
         "num_rows": total_rows,
@@ -372,7 +417,6 @@ def _capture_stats(df: DataFrame,
     }
 
     def add_var_metadata(variable, var_type, group_label):
-        mapped_var = mapper.get(variable, variable)
         if variable in continuous_vars:
             var_ref_edges = reference_edges[continuous_vars.index(variable)]
             data_struct[var_type][variable] = _process_continuous_var(
@@ -380,14 +424,14 @@ def _capture_stats(df: DataFrame,
                 stats,
                 var_ref_edges,
                 group_label,
-                mapped_var,
+                variable,
                 importance.get(variable, None))
 
         else:
             data_struct[var_type][variable] = _process_categorical_var(
                 frequencies,
                 group_label,
-                mapped_var,
+                variable,
                 importance.get(variable, None),
                 category_labels.get(variable, {}),
                 category_ordinals.get(variable, False),
@@ -395,10 +439,10 @@ def _capture_stats(df: DataFrame,
             )
 
     for var in features:
-        add_var_metadata(var, "features", feature_group)
+        add_var_metadata(var, "features", feature_metadata_group)
 
-    for var in predictors:
-        add_var_metadata(var, "predictors", predictor_group)
+    for var in targets:
+        add_var_metadata(var, "predictors", feature_metadata_group)
 
     return data_struct
 
@@ -440,7 +484,8 @@ def parse_scoring_observation(features_df: pd.DataFrame, predicted_df: pd.DataFr
         for name, value in data[var_type].items():
             if "type" in value and value["type"] == "categorical":
                 data[var_type][name]["value"] = values[name].values[0]
-            elif "statistics" in value and "histogram" in value["statistics"] and "values" in value["statistics"]["histogram"]:
+            elif "statistics" in value and "histogram" in value["statistics"] and "values" in value["statistics"][
+                "histogram"]:
                 data[var_type][name]["statistics"]["histogram"]["values"] = values[name].values[0]
 
         return data_struct
@@ -456,13 +501,13 @@ def parse_scoring_observation(features_df: pd.DataFrame, predicted_df: pd.DataFr
 
 def record_training_stats(df: DataFrame,
                           features: List,
-                          predictors: List,
-                          categorical: List,
+                          targets: List = [],
+                          categorical: List = [],
                           category_labels: Dict = {},
                           category_ordinals: Dict = {},
                           importance: Dict = {},
-                          feature_group="default",
-                          predictor_group="default"):
+                          context: ModelContext = None,
+                          **kwargs):
     """
     Record the training statistics.
 
@@ -471,42 +516,61 @@ def record_training_stats(df: DataFrame,
 
         record_training_stats(pima,
                    features=["TwoHourSerIns", "Age"],
-                   predictors=["HasDiabetes"],
+                   targets=["HasDiabetes"],
                    categorical=["HasDiabetes"],
-                   importance={"Age": 0.9, "TwoHourSerIns": 0.1},
-                   category_labels={"HasDiabetes": {0: "false", 1: "true"}})
+                   importance={"Age": 0.9, "TwoHourSerIns": 0.1})
 
     :param df:
     :param features:
-    :param predictors:
+    :param targets:
     :param categorical:
     :param category_labels:
     :param category_ordinals:
     :param importance:
-    :param feature_group:
-    :param predictor_group:
-    :return:
+    :param context: ModelContext
+    :return data_stats: Dict of statistics
     """
+    logger.info("Computing training dataset statistics")
 
-    data_stats = _capture_stats(df,
-                                features,
-                                predictors,
-                                categorical,
-                                category_labels,
-                                category_ordinals,
-                                {},
-                                importance,
-                                feature_group,
-                                predictor_group)
+    if not features:
+        raise ValueError("One or more features must be provided")
 
-    with open("artifacts/output/data_stats.json", 'w+') as f:
+    if not targets:
+        # backward compatibility due to rename of predictors to targets in api
+        if "predictors" in kwargs:
+            targets = kwargs["predictors"]
+        else:
+            raise ValueError("One or more targets must be provided")
+
+    feature_metadata_fqtn = None
+    feature_metadata_group = None
+    data_stats_filename = "artifacts/output/data_stats.json"
+
+    if context:
+        feature_metadata_fqtn = context.dataset_info.get_feature_metadata_fqtn()
+        feature_metadata_group = context.dataset_info.feature_metadata_monitoring_group
+        data_stats_filename = os.path.join(context.artifact_output_path, "data_stats.json")
+
+    data_stats = _capture_stats(df=df,
+                                features=features,
+                                targets=targets,
+                                categorical=categorical,
+                                category_labels=category_labels,
+                                category_ordinals=category_ordinals,
+                                importance=importance,
+                                feature_metadata_fqtn=feature_metadata_fqtn,
+                                feature_metadata_group=feature_metadata_group)
+
+    with open(data_stats_filename, 'w+') as f:
         json.dump(data_stats, f, indent=2, cls=_NpEncoder)
 
+    return data_stats
+
 
 def record_evaluation_stats(features_df: DataFrame,
                             predicted_df: DataFrame,
                             importance=None,
-                            mapper: Dict = {}):
+                            context: ModelContext = None):
     """
     Record the evaluation statistics.
 
@@ -517,25 +581,47 @@ def record_evaluation_stats(features_df: DataFrame,
         record_evaluation_stats(
                    features_df,
                    predicted_df,
-                   importance={"TwoHourSerIns": 0.1, "Age": 0.2})
+                   context)
 
     :param features_df: df with the input features for scoring
     :param predicted_df: df with the predicted values. optionally include the predict confidence
     :param importance: (optional) feature importance
-    :param mapper: (optional) Dictionary with the column names from the model as keys and the new names from the provided dataframes as values. E.g. {"target": "prediction"}
-    :return:
+    :param context: ModelContext
+    :return data_stats: Dict of statistics
     """
+    logger.info("Computing evaluation dataset statistics")
 
-    data_struct = _parse_scoring_stats(features_df, predicted_df, importance, mapper=mapper)
+    feature_metadata_fqtn = None
+    feature_metadata_group = None
+    output_data_stats_filename = "artifacts/output/data_stats.json"
+    input_data_stats_filename = "artifacts/input/data_stats.json"
+
+    if context:
+        feature_metadata_fqtn = context.dataset_info.get_feature_metadata_fqtn()
+        feature_metadata_group = context.dataset_info.feature_metadata_monitoring_group
+        output_data_stats_filename = os.path.join(context.artifact_output_path, "data_stats.json")
+        input_data_stats_filename = os.path.join(context.artifact_input_path, "data_stats.json")
+
+    with open(input_data_stats_filename, 'r') as f:
+        training_data_stats = json.load(f)
+
+    data_stats = _parse_scoring_stats(features_df=features_df,
+                                      predicted_df=predicted_df,
+                                      data_stats=training_data_stats,
+                                      importance_dict=importance,
+                                      feature_metadata_fqtn=feature_metadata_fqtn,
+                                      feature_metadata_group=feature_metadata_group)
 
     # for evaluation, the core will do it (we may change this later to unify)..
-    with open("artifacts/output/data_stats.json", 'w+') as f:
-        json.dump(data_struct, f, indent=2, cls=_NpEncoder)
+    with open(output_data_stats_filename, 'w+') as f:
+        json.dump(data_stats, f, indent=2, cls=_NpEncoder)
+
+    return data_stats
 
 
 def record_scoring_stats(features_df: DataFrame,
                          predicted_df: DataFrame,
-                         mapper: Dict = {}):
+                         context: ModelContext = None):
     """
     Record the scoring statistics.
 
@@ -549,15 +635,59 @@ def record_scoring_stats(features_df: DataFrame,
         record_scoring_stats(
                    features_df,
                    predicted_df,
-                   mapper)
+                   context)
 
     :param features_df: df with the input features for scoring
     :param predicted_df: df with the predicted values. optionally include the predict confidence
-    :param mapper: (optional) Dictionary with the column names from the model as keys and the new names from the provided dataframes as values. E.g. {"target": "prediction"}
-    :return:
+    :param context: ModelContext
+    :return data_stats: Dict of statistics
     """
-    data_struct = _parse_scoring_stats(features_df, predicted_df, mapper=mapper)
+    logger.info("Computing scoring dataset statistics")
+
+    feature_metadata_fqtn = None
+    feature_metadata_group = None
+    input_data_stats_filename = "artifacts/input/data_stats.json"
+
+    if context:
+        feature_metadata_fqtn = context.dataset_info.get_feature_metadata_fqtn()
+        feature_metadata_group = context.dataset_info.feature_metadata_monitoring_group
+        input_data_stats_filename = os.path.join(context.artifact_input_path, "data_stats.json")
+
+    with open(input_data_stats_filename, 'r') as f:
+        training_data_stats = json.load(f)
+
+    data_stats = _parse_scoring_stats(features_df=features_df,
+                                      predicted_df=predicted_df,
+                                      data_stats=training_data_stats,
+                                      feature_metadata_fqtn=feature_metadata_fqtn,
+                                      feature_metadata_group=feature_metadata_group)
 
     # Another possibility could be to make an api call to the core to publish
     # it (but for now let's implement it on the scorer side)
-    publish_data_stats(data_struct)
+    publish_data_stats(data_stats)
+
+    return data_stats
+
+
+def compute_continuous_stats(features_df: DataFrame, continuous_features: List, **kwargs):
+    dtypes = {r[0].lower(): r[1] for r in features_df.dtypes._column_names_and_types}
+
+    stats = valib.Statistics(data=features_df, columns=continuous_features, stats_options="all")
+    stats = stats.result.to_pandas().reset_index()
+    reference_edges = _compute_continuous_edges(continuous_features, stats, dtypes)
+    edges_dict = dict(zip(continuous_features, reference_edges))
+    column_stats = {f.lower(): {'edges': edges_dict[f]} for f in edges_dict.keys()}
+    return column_stats
+
+
+def compute_categorical_stats(features_df: DataFrame, categorical_features: List, **kwargs):
+    statistics = valib.Frequency(data=features_df, columns=categorical_features)
+    statistics = statistics.result.to_pandas().reset_index()
+    statistics = statistics.drop(statistics.columns.difference(["xcol", "xval", "xpct"]), axis=1)
+    statistics = statistics.groupby('xcol').apply(lambda x: dict(zip(x['xval'], x['xpct']))).to_dict()
+    column_stats = {}
+    for var in categorical_features:
+        column_stats[var.lower()] = {
+            'frequencies': {k: float(Decimal("{:.2f}".format(float(v)))) for k, v in statistics[var].items()}}
+
+    return column_stats
